/*********************************************************************
 *
 * This is based on code created by Peter Harvey,
 * (pharvey@codebydesign.com).
 *
 * Modified and extended by Nick Gorham
 * (nick@easysoft.com).
 *
 * Any bugs or problems should be considered the fault of Nick and not
 * Peter.
 *
 * copyright (c) 1999 Nick Gorham
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **********************************************************************
 *
 * $Id: SQLConnect.c,v 1.26 2001/06/25 12:55:15 nick Exp $
 *
 * $Log: SQLConnect.c,v $
 * Revision 1.26  2001/06/25 12:55:15  nick
 *
 * Fix threading problem with multiple ENV's
 *
 * Revision 1.25  2001/06/13 11:23:11  nick
 *
 * Fix a couple of portability problems
 *
 * Revision 1.24  2001/05/31 16:05:55  nick
 *
 * Fix problems with postgres closing local sockets
 * Make odbctest build with QT 3 (it doesn't work due to what I think are bugs
 * in QT 3)
 * Fix a couple of problems in the cursor lib
 *
 * Revision 1.23  2001/05/23 11:44:44  nick
 *
 * Fix typo
 *
 * Revision 1.22  2001/05/09 11:56:47  nick
 *
 * Add support for libtool 1.4
 *
 * Revision 1.21  2001/04/18 15:03:37  nick
 *
 * Fix problem when going to DB2 unicode driver
 *
 * Revision 1.20  2001/04/16 22:35:10  nick
 *
 * More tweeks to the AutoTest code
 *
 * Revision 1.19  2001/04/16 15:41:24  nick
 *
 * Fix some problems calling non existing error funcs
 *
 * Revision 1.18  2001/04/12 17:43:36  nick
 *
 * Change logging and added autotest to odbctest
 *
 * Revision 1.17  2001/04/04 11:30:38  nick
 *
 * Fix a memory leak in Postgre7.1
 * Fix a problem with timed out pooled connections
 * Add time to live option for pooled connections
 *
 * Revision 1.16  2001/04/03 16:34:12  nick
 *
 * Add support for strangly broken unicode drivers
 *
 * Revision 1.15  2001/03/30 08:35:39  nick
 *
 * Fix a couple of pooling problems
 *
 * Revision 1.14  2001/03/02 14:24:23  nick
 *
 * Fix thread detection for Solaris
 *
 * Revision 1.13  2001/02/12 11:20:22  nick
 *
 * Add supoort for calling SQLDriverLoad and SQLDriverUnload
 *
 * Revision 1.12  2000/12/31 20:30:54  nick
 *
 * Add UNICODE support
 *
 * Revision 1.11  2000/12/18 13:02:13  nick
 *
 * More buf fixes
 *
 * Revision 1.10  2000/12/17 11:02:37  nick
 *
 * Fix extra '*'
 *
 * Revision 1.9  2000/12/17 11:00:32  nick
 *
 * Add thread safe bits to pooling
 *
 * Revision 1.8  2000/12/14 18:10:19  nick
 *
 * Add connection pooling
 *
 * Revision 1.7  2000/11/29 11:26:18  nick
 *
 * Add unicode bits
 *
 * Revision 1.6  2000/11/22 18:35:43  nick
 *
 * Check input handle before touching output handle
 *
 * Revision 1.5  2000/11/22 17:19:32  nick
 *
 * Fix tracing problem in SQLConnect
 *
 * Revision 1.4  2000/11/14 10:15:27  nick
 *
 * Add test for localtime_r
 *
 * Revision 1.3  2000/10/25 08:58:55  nick
 *
 * Fix crash when null server and SQL_NTS is passed in
 *
 * Revision 1.2  2000/10/13 15:18:49  nick
 *
 * Change string length parameter from SQLINTEGER to SQLSMALLINT
 *
 * Revision 1.1.1.1  2000/09/04 16:42:52  nick
 * Imported Sources
 *
 * Revision 1.30  2000/07/28 14:57:29  ngorham
 *
 * Don't copy the function pointers for ColAttribute, ColAttributes just
 * set can_supply
 *
 * Revision 1.29  2000/06/27 17:34:09  ngorham
 *
 * Fix a problem when the second part of the connect failed a seg fault
 * was generated in the error reporting
 *
 * Revision 1.28  2001/05/26 19:11:37  ngorham
 *
 * Add SQLCopyDesc functionality and fix bug that was stopping messages
 * coming out of SQLConnect
 *
 * Revision 1.27  2000/05/21 21:49:19  ngorham
 *
 * Assorted fixes
 *
 * Revision 1.26  2000/04/27 20:49:03  ngorham
 *
 * Fixes to work with Star Office 5.2
 *
 * Revision 1.25  2000/04/19 22:00:57  ngorham
 *
 * We can always supply SQLGetFunctions
 *
 * Revision 1.24  2000/03/11 15:55:47  ngorham
 *
 * A few more changes and bug fixes (see NEWS)
 *
 * Revision 1.23  2000/02/25 00:02:00  ngorham
 *
 * Add a patch to support IBM DB2, and Solaris threads
 *
 * Revision 1.22  2000/02/02 07:55:20  ngorham
 *
 * Add flag to disable SQLFetch -> SQLExtendedFetch mapping
 *
 * Revision 1.21  1999/12/28 15:05:00  ngorham
 *
 * Fix bug that caused StarOffice to fail. A SQLConnect, SQLDisconnect,
 * followed by another SQLConnect on the same DBC would fail.
 *
 * Revision 1.20  1999/12/17 09:40:30  ngorham
 *
 * Change a error return from HY004 to IM004
 *
 * Revision 1.19  1999/12/14 19:02:25  ngorham
 *
 * Mask out the password fields in the logging
 *
 * Revision 1.18  1999/11/13 23:40:58  ngorham
 *
 * Alter the way DM logging works
 * Upgrade the Postgres driver to 6.4.6
 *
 * Revision 1.17  1999/11/10 03:51:33  ngorham
 *
 * Update the error reporting in the DM to enable ODBC 3 and 2 calls to
 * work at the same time
 *
 * Revision 1.16  1999/10/24 23:54:17  ngorham
 *
 * First part of the changes to the error reporting
 *
 * Revision 1.15  1999/10/14 06:49:24  ngorham
 *
 * Remove @all_includes@ from Drivers/MiniSQL/Makefile.am
 *
 * Revision 1.14  1999/10/09 00:15:58  ngorham
 *
 * Add mapping from SQL_TYPE_X to SQL_X and SQL_C_TYPE_X to SQL_C_X
 * when the driver is a ODBC 2 one
 *
 * Revision 1.13  1999/10/07 20:39:25  ngorham
 *
 * Added .cvsignore files and fixed a couple of bugs in the DM
 *
 * Revision 1.12  1999/10/06 07:10:46  ngorham
 *
 * As the book says check dlerror after a dl func
 *
 * Revision 1.11  1999/10/06 07:01:25  ngorham
 *
 * Added more support for non linux platforms
 *
 * Revision 1.10  1999/09/26 18:55:03  ngorham
 *
 * Fixed a problem where the cursor lib was being used by default
 *
 * Revision 1.9  1999/09/24 22:54:52  ngorham
 *
 * Fixed some unchanged dlopen,dlsym,dlclose functions
 *
 * Revision 1.8  1999/09/21 22:34:24  ngorham
 *
 * Improve performance by removing unneeded logging calls when logging is
 * disabled
 *
 * Revision 1.7  1999/09/20 21:46:49  ngorham
 *
 * Added support for libtld dlopen replace
 *
 * Revision 1.6  1999/09/19 22:24:33  ngorham
 *
 * Added support for the cursor library
 *
 * Revision 1.5  1999/08/03 21:47:39  shandyb
 * Moving to automake: changed files in DriverManager
 *
 * Revision 1.4  1999/07/10 21:10:15  ngorham
 *
 * Adjust error sqlstate from driver manager, depending on requested
 * version (ODBC2/3)
 *
 * Revision 1.3  1999/07/04 21:05:07  ngorham
 *
 * Add LGPL Headers to code
 *
 * Revision 1.2  1999/06/30 23:56:54  ngorham
 *
 * Add initial thread safety code
 *
 * Revision 1.1.1.1  1999/05/29 13:41:05  sShandyb
 * first go at it
 *
 * Revision 1.4  1999/06/07 01:29:30  pharvey
 * *** empty log message ***
 *
 * Revision 1.3  1999/06/02 20:12:10  ngorham
 *
 * Fixed botched log entry, and removed the dos \r from the sql header files.
 *
 * Revision 1.2  1999/06/02 19:57:20  ngorham
 *
 * Added code to check if a attempt is being made to compile with a C++
 * Compiler, and issue a message.
 * Start work on the ODBC2-3 conversions.
 *
 * Revision 1.1.1.1  1999/05/27 18:23:17  pharvey
 * Imported sources
 *
 * Revision 1.7  1999/05/09 23:27:11  nick
 * All the API done now
 *
 * Revision 1.6  1999/05/04 22:41:12  nick
 * and another night ends
 *
 * Revision 1.5  1999/05/03 19:50:43  nick
 * Another check point
 *
 * Revision 1.4  1999/04/30 16:22:47  nick
 * Another checkpoint
 *
 * Revision 1.3  1999/04/29 21:40:58  nick
 * End of another night :-)
 *
 * Revision 1.2  1999/04/29 20:47:37  nick
 * Another checkpoint
 *
 * Revision 1.1  1999/04/25 23:06:11  nick
 * Initial revision
 *
 *
 **********************************************************************/

#include "drivermanager.h"

static char const rcsid[]= "$RCSfile: SQLConnect.c,v $ $Revision: 1.26 $";

#define CURSOR_LIB      "libodbccr.so"

/*
 * structure to contain the loaded lib entry points
 */

static struct driver_func  template_func[] =
{
    /* 00 */ { SQL_API_SQLALLOCCONNECT,      "SQLAllocConnect", SQLAllocConnect },
    /* 01 */ { SQL_API_SQLALLOCENV,          "SQLAllocEnv", SQLAllocEnv  },
    /* 02 */ { SQL_API_SQLALLOCHANDLE,       "SQLAllocHandle", SQLAllocHandle },
    /* 03 */ { SQL_API_SQLALLOCSTMT,         "SQLAllocStmt", SQLAllocStmt },
    /* 04 */ { SQL_API_SQLALLOCHANDLESTD,    "SQLAllocHandleStd", SQLAllocHandleStd },
    /* 05 */ { SQL_API_SQLBINDCOL,           "SQLBindCol", SQLBindCol },
    /* 06 */ { SQL_API_SQLBINDPARAM,         "SQLBindParam", SQLBindParam },
    /* 07 */ { SQL_API_SQLBINDPARAMETER,     "SQLBindParameter", SQLBindParameter },
    /* 08 */ { SQL_API_SQLBROWSECONNECT,     "SQLBrowseConnect", 
                SQLBrowseConnect, SQLBrowseConnectW },
    /* 09 */ { SQL_API_SQLBULKOPERATIONS,    "SQLBulkOperations", SQLBulkOperations },
    /* 10 */ { SQL_API_SQLCANCEL,            "SQLCancel", SQLCancel },
    /* 11 */ { SQL_API_SQLCLOSECURSOR,       "SQLCloseCursor", SQLCloseCursor },
    /* 12 */ { SQL_API_SQLCOLATTRIBUTE,      "SQLColAttribute", 
                SQLColAttribute, SQLColAttributeW },
    /* 13 */ { SQL_API_SQLCOLATTRIBUTES,     "SQLColAttributes", 
                SQLColAttributes, SQLColAttributesW },
    /* 14 */ { SQL_API_SQLCOLUMNPRIVILEGES,  "SQLColumnPrivileges", 
                SQLColumnPrivileges, SQLColumnPrivilegesW },
    /* 15 */ { SQL_API_SQLCOLUMNS,           "SQLColumns", 
                SQLColumns, SQLColumnsW },
    /* 16 */ { SQL_API_SQLCONNECT,           "SQLConnect", 
                SQLConnect, SQLConnectW },
    /* 17 */ { SQL_API_SQLCOPYDESC,          "SQLCopyDesc", SQLCopyDesc },
    /* 18 */ { SQL_API_SQLDATASOURCES,       "SQLDataSources", 
                SQLDataSources, SQLDataSourcesW },
    /* 19 */ { SQL_API_SQLDESCRIBECOL,       "SQLDescribeCol", 
                SQLDescribeCol, SQLDescribeColW },
    /* 20 */ { SQL_API_SQLDESCRIBEPARAM,     "SQLDescribeParam", SQLDescribeParam },
    /* 21 */ { SQL_API_SQLDISCONNECT,        "SQLDisconnect", SQLDisconnect },
    /* 22 */ { SQL_API_SQLDRIVERCONNECT,     "SQLDriverConnect", 
                SQLDriverConnect, SQLDriverConnectW },
    /* 23 */ { SQL_API_SQLDRIVERS,           "SQLDrivers", 
                SQLDrivers, SQLDriversW },
    /* 24 */ { SQL_API_SQLENDTRAN,           "SQLEndTran", SQLEndTran },
    /* 25 */ { SQL_API_SQLERROR,             "SQLError", 
                SQLError, SQLErrorW },
    /* 26 */ { SQL_API_SQLEXECDIRECT,        "SQLExecDirect", 
                SQLExecDirect, SQLExecDirectW },
    /* 27 */ { SQL_API_SQLEXECUTE,           "SQLExecute", SQLExecute },
    /* 28 */ { SQL_API_SQLEXTENDEDFETCH,     "SQLExtendedFetch", SQLExtendedFetch },
    /* 29 */ { SQL_API_SQLFETCH,             "SQLFetch", SQLFetch },
    /* 30 */ { SQL_API_SQLFETCHSCROLL,       "SQLFetchScroll", SQLFetchScroll },
    /* 31 */ { SQL_API_SQLFOREIGNKEYS,       "SQLForeignKeys", 
                SQLForeignKeys, SQLForeignKeysW },
    /* 32 */ { SQL_API_SQLFREEENV,           "SQLFreeEnv", SQLFreeEnv },
    /* 33 */ { SQL_API_SQLFREEHANDLE,        "SQLFreeHandle", SQLFreeHandle },
    /* 34 */ { SQL_API_SQLFREESTMT,          "SQLFreeStmt", SQLFreeStmt },
    /* 35 */ { SQL_API_SQLFREECONNECT,       "SQLFreeConnect", SQLFreeConnect },
    /* 36 */ { SQL_API_SQLGETCONNECTATTR,    "SQLGetConnectAttr", 
                SQLGetConnectAttr, SQLGetConnectAttrW },
    /* 37 */ { SQL_API_SQLGETCONNECTOPTION,  "SQLGetConnectOption", 
                SQLGetConnectOption, SQLGetConnectOptionW },
    /* 38 */ { SQL_API_SQLGETCURSORNAME,     "SQLGetCursorName", 
                SQLGetCursorName, SQLGetCursorNameW },
    /* 39 */ { SQL_API_SQLGETDATA,           "SQLGetData", SQLGetData },
    /* 40 */ { SQL_API_SQLGETDESCFIELD,      "SQLGetDescField", 
                SQLGetDescField, SQLGetDescFieldW },
    /* 41 */ { SQL_API_SQLGETDESCREC,        "SQLGetDescRec", 
                SQLGetDescRec, SQLGetDescRecW },
    /* 42 */ { SQL_API_SQLGETDIAGFIELD,      "SQLGetDiagField", 
                SQLGetDiagField, SQLGetDiagFieldW },
    /* 43 */ { SQL_API_SQLGETENVATTR,        "SQLGetEnvAttr", SQLGetEnvAttr },
    /* 44 */ { SQL_API_SQLGETFUNCTIONS,      "SQLGetFunctions", SQLGetFunctions },
    /* 45 */ { SQL_API_SQLGETINFO,           "SQLGetInfo", 
                SQLGetInfo, SQLGetInfoW },
    /* 46 */ { SQL_API_SQLGETSTMTATTR,       "SQLGetStmtAttr", 
                SQLGetStmtAttr, SQLGetStmtAttrW },
    /* 47 */ { SQL_API_SQLGETSTMTOPTION,     "SQLGetStmtOption", SQLGetStmtOption },
    /* 48 */ { SQL_API_SQLGETTYPEINFO,       "SQLGetTypeInfo", 
                SQLGetTypeInfo, SQLGetTypeInfoW },
    /* 49 */ { SQL_API_SQLMORERESULTS,       "SQLMoreResults", SQLMoreResults },
    /* 50 */ { SQL_API_SQLNATIVESQL,         "SQLNativeSql", 
                SQLNativeSql, SQLNativeSqlW },
    /* 51 */ { SQL_API_SQLNUMPARAMS,         "SQLNumParams", SQLNumParams },
    /* 52 */ { SQL_API_SQLNUMRESULTCOLS,     "SQLNumResultCols", SQLNumResultCols },
    /* 53 */ { SQL_API_SQLPARAMDATA,         "SQLParamData", SQLParamData },
    /* 54 */ { SQL_API_SQLPARAMOPTIONS,      "SQLParamOptions", SQLParamOptions },
    /* 55 */ { SQL_API_SQLPREPARE,           "SQLPrepare", 
                SQLPrepare, SQLPrepareW },
    /* 56 */ { SQL_API_SQLPRIMARYKEYS,       "SQLPrimaryKeys", 
                SQLPrimaryKeys, SQLPrimaryKeysW },
    /* 57 */ { SQL_API_SQLPROCEDURECOLUMNS,  "SQLProcedureColumns", 
                SQLProcedureColumns, SQLProcedureColumnsW },
    /* 58 */ { SQL_API_SQLPROCEDURES,        "SQLProcedures", 
                SQLProcedures, SQLProceduresW },
    /* 59 */ { SQL_API_SQLPUTDATA,           "SQLPutData", SQLPutData },
    /* 60 */ { SQL_API_SQLROWCOUNT,          "SQLRowCount", SQLRowCount },
    /* 61 */ { SQL_API_SQLSETCONNECTATTR,    "SQLSetConnectAttr", 
                SQLSetConnectAttr, SQLSetConnectAttrW },
    /* 62 */ { SQL_API_SQLSETCONNECTOPTION,  "SQLSetConnectOption", 
                SQLSetConnectOption, SQLSetConnectOptionW },
    /* 63 */ { SQL_API_SQLSETCURSORNAME,     "SQLSetCursorName", 
                SQLSetCursorName, SQLSetCursorNameW },
    /* 64 */ { SQL_API_SQLSETDESCFIELD,      "SQLSetDescField", 
                SQLSetDescField, SQLSetDescFieldW },
    /* 65 */ { SQL_API_SQLSETDESCREC,        "SQLSetDescRec", SQLSetDescRec },
    /* 66 */ { SQL_API_SQLSETENVATTR,        "SQLSetEnvAttr", SQLSetEnvAttr },
    /* 67 */ { SQL_API_SQLSETPARAM,          "SQLSetParam", SQLSetParam },
    /* 68 */ { SQL_API_SQLSETPOS,            "SQLSetPos", SQLSetPos },
    /* 69 */ { SQL_API_SQLSETSCROLLOPTIONS,  "SQLSetScrollOptions", SQLSetScrollOptions },
    /* 70 */ { SQL_API_SQLSETSTMTATTR,       "SQLSetStmtAttr", 
                SQLSetStmtAttr, SQLSetStmtAttrW },
    /* 71 */ { SQL_API_SQLSETSTMTOPTION,     "SQLSetStmtOption", SQLSetStmtOption },
    /* 72 */ { SQL_API_SQLSPECIALCOLUMNS,    "SQLSpecialColumns", 
                SQLSpecialColumns, SQLSpecialColumnsW },
    /* 73 */ { SQL_API_SQLSTATISTICS,        "SQLStatistics", 
                SQLStatistics, SQLStatisticsW },
    /* 74 */ { SQL_API_SQLTABLEPRIVILEGES,   "SQLTablePrivileges", 
                SQLTablePrivileges, SQLTablePrivilegesW },
    /* 75 */ { SQL_API_SQLTABLES,            "SQLTables", 
                SQLTables, SQLTablesW },
    /* 76 */ { SQL_API_SQLTRANSACT,          "SQLTransact", SQLTransact },
    /* 77 */ { SQL_API_SQLGETDIAGREC,        "SQLGetDiagRec", 
                SQLGetDiagRec, SQLGetDiagRecW },
};

/*
 * connection pooling stuff
 */

CPOOL *pool_head = NULL;
int pooling_enabled = 0;

/*
 * open the library, extract the names, and do setup
 * before the actual connect.
 */

int __connect_part_one( DMHDBC connection, char *driver_lib, char *driver_name )
{
    int i;
    int ret;
    SQLCHAR s0[ 20 ];
    int threading_level;
    char threading_string[ 50 ];
    char mapping_string[ 50 ];
    char fake_string[ 50 ];
    int fake_unicode;

    /*
     * check to see if we want to alter the default threading level
     * before opening the lib
     */

    SQLGetPrivateProfileString( driver_name, "Threading", "3",
				threading_string, sizeof( threading_string ), 
                "ODBCINST.INI" );

    threading_level = atoi( threading_string );

    if ( threading_level >= 0 && threading_level <= 3 )
    {
        dbc_change_thread_support( connection, threading_level );
    }

    /*
     * do we want to disable the SQLFetch -> SQLExtendedFetch 
     * mapping ?
     */

    SQLGetPrivateProfileString( driver_name, "ExFetchMapping", "1",
				mapping_string, sizeof( mapping_string ), 
                "ODBCINST.INI" );

    connection -> ex_fetch_mapping = atoi( mapping_string );

    /*
     * do we want to keep hold of the lib handle, DB2 fails if we close
     */

    SQLGetPrivateProfileString( driver_name, "DontDLClose", "0",
				mapping_string, sizeof( mapping_string ), 
                "ODBCINST.INI" );

    connection -> dont_dlclose = atoi( mapping_string ) != 0;

    /*
     * can we pool this one
     */

    SQLGetPrivateProfileString( driver_name, "CPTimeout", "0",
				mapping_string, sizeof( mapping_string ), 
                "ODBCINST.INI" );

    connection -> pooling_timeout = atoi( mapping_string );

    /*
     * have we got a time-to-live value for the pooling
     */

    SQLGetPrivateProfileString( driver_name, "CPTimeToLive", "0",
				mapping_string, sizeof( mapping_string ), 
                "ODBCINST.INI" );

    connection -> ttl = atoi( mapping_string );

    /*
     * if pooling then leave the dlopen
     */

    if ( connection -> pooling_timeout > 0 )
    {
        connection -> dont_dlclose = 1;
    }

    SQLGetPrivateProfileString( driver_name, "FakeUnicode", "0",
				fake_string, sizeof( fake_string ), 
                "ODBCINST.INI" );

    fake_unicode = atoi( fake_string );

    /*
     * initialize libtool
     */

    lt_dlinit();

    /*
     * open the lib
     */

    connection -> driver_env = (SQLHANDLE)NULL;
    connection -> driver_dbc = (SQLHANDLE)NULL;
    connection -> functions = NULL;
    connection -> dl_handle = NULL;

    if ( !(connection -> dl_handle = lt_dlopen( driver_lib )))
    {
        char txt[ 256 ];

        sprintf( txt, "Can't open lib '%s' : %s", 
                driver_lib, lt_dlerror());

        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                txt );

        __post_internal_error( &connection -> error,
                ERROR_IM003, txt,
                connection -> environment -> requested_version );
        return 0;
    }

    /*
     * try and extract the ini and fini functions, and call ini if its 
     * found
     */

    connection -> ini_func.func =
            lt_dlsym( connection -> dl_handle,
                    ODBC_INI_FUNCTION );

    connection -> fini_func.func =
            lt_dlsym( connection -> dl_handle,
                    ODBC_FINI_FUNCTION );

    if ( connection -> ini_func.func )
    {
        connection -> ini_func.func();
    }

    /*
     * extract all the function entry points
     */
    if ( !(connection -> functions = malloc( sizeof( template_func ))))
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: IM001" );

        __post_internal_error( &connection -> error,
                ERROR_HY001, NULL,
                connection -> environment -> requested_version );
        return 0;
    }

    memcpy( connection -> functions, template_func,
            sizeof( template_func ));

    for ( i = 0;
            i < sizeof( template_func ) / sizeof( template_func[ 0 ] );
            i ++ )
    {
        char name[ 128 ];

        connection -> functions[ i ].func =
            lt_dlsym( connection -> dl_handle,
                    connection -> functions[ i ].name );

        if ( connection -> functions[ i ].dm_funcW )
        {
            /*
             * get ANSI version from driver
             */

            if ( fake_unicode )
            {
                sprintf( name, "%sW", connection -> functions[ i ].name );
            }
            else
            {
                sprintf( name, "%sA", connection -> functions[ i ].name );
            }
            connection -> functions[ i ].funcA =
                lt_dlsym( connection -> dl_handle, name );

            if ( connection -> functions[ i ].funcA &&
                    !connection -> functions[ i ].func )
            {
                connection -> functions[ i ].func =
                    connection -> functions[ i ].funcA;
            }
            else if ( connection -> functions[ i ].func &&
                !connection -> functions[ i ].funcA )
            {
                connection -> functions[ i ].funcA =
                    connection -> functions[ i ].func;
            }

            /*
             * get UNICODE version from driver
             */

            sprintf( name, "%sW", connection -> functions[ i ].name );
            connection -> functions[ i ].funcW =
                lt_dlsym( connection -> dl_handle, name );
        }
        else
        {
            connection -> functions[ i ].funcA = 
                connection -> functions[ i ].funcW = NULL;
        }

        /*
         * blank out ones that are in the DM to fix a big 
         * with glib 2.0.6
         */

		if ( connection -> functions[ i ].func &&
			(void*)connection -> functions[ i ].func == 
            (void*)connection -> functions[ i ].dm_func )
		{	
			connection -> functions[ i ].func = NULL;
		}

		if ( connection -> functions[ i ].funcW &&
			(void*)connection -> functions[ i ].funcW == 
            (void*)connection -> functions[ i ].dm_funcW )
		{	
			connection -> functions[ i ].funcW = NULL;
		}

        connection -> functions[ i ].can_supply =
            ( connection -> functions[ i ].func != NULL ) || 
              ( connection -> functions[ i ].funcW != NULL );
    }

    /*
     * allocate a env handle
     */

    if ( CHECK_SQLALLOCHANDLE( connection ))
    {
        ret = SQLALLOCHANDLE( connection,
                SQL_HANDLE_ENV,
                SQL_NULL_HENV,
                &connection -> driver_env,
                connection );
    }
    else if ( CHECK_SQLALLOCENV( connection ))
    {
        ret = SQLALLOCENV( connection,
                &connection -> driver_env );
    }
    else
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: IM004" );

        __post_internal_error( &connection -> error,
                ERROR_IM004, NULL,
                connection -> environment -> requested_version );
        return 0;
    }

    if ( ret )
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: IM004" );

        __post_internal_error( &connection -> error,
                ERROR_IM004, NULL,
                connection -> environment -> requested_version );
        return 0;
    }

    /*
     * if it looks like a 3.x driver, try setting the interface type
     * to 3.x
     */
    if ( CHECK_SQLSETENVATTR( connection ))
    {
        ret = SQLSETENVATTR( connection,
                connection -> driver_env,
                SQL_ATTR_ODBC_VERSION,
                connection -> environment -> requested_version,
                0 );

        /*
         * if it don't set then assume a 2.x driver
         */

        if ( ret )
        {
            connection -> driver_version = SQL_OV_ODBC2;
        }
        else
        {
            if ( CHECK_SQLGETENVATTR( connection ))
            {
                SQLINTEGER actual_version;

                ret = SQLGETENVATTR( connection,
                    connection -> driver_env,
                    SQL_ATTR_ODBC_VERSION,
                    &actual_version,
                    0,
                    NULL );

                if ( !ret )
                {
                    connection -> driver_version = actual_version;
                }
                else
                {
                    connection -> driver_version =
                        connection -> environment -> requested_version;
                }
            }
            else
            {
                connection -> driver_version =
                    connection -> environment -> requested_version;
            }
        }
    }
    else
    {
        connection -> driver_version = SQL_OV_ODBC2;
    }

    /*
     * allocate a connection handle
     */
    if ( connection -> driver_version == SQL_OV_ODBC3 )
    {
        if ( CHECK_SQLALLOCHANDLE( connection ))
        {
            ret = SQLALLOCHANDLE( connection,
                    SQL_HANDLE_DBC,
                    connection -> driver_env,
                    &connection -> driver_dbc,
                    connection );

            if ( ret )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: IM005" );

                __post_internal_error( &connection -> error,
                        ERROR_IM005, NULL,
                        connection -> environment -> requested_version );
                return 0;
            }
        }
        else if ( CHECK_SQLALLOCCONNECT( connection ))
        {
            ret = SQLALLOCCONNECT( connection,
                    connection -> driver_env,
                    &connection -> driver_dbc );

            if ( ret )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: IM005" );

                __post_internal_error( &connection -> error,
                        ERROR_IM005, NULL,
                        connection -> environment -> requested_version );
                return 0;
            }
        }
        else
        {
            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    "Error: IM005" );

            __post_internal_error( &connection -> error,
                    ERROR_IM005, NULL,
                    connection -> environment -> requested_version );
            return 0;
        }
    }
    else
    {
        if ( CHECK_SQLALLOCCONNECT( connection ))
        {
            ret = SQLALLOCCONNECT( connection,
                    connection -> driver_env,
                    &connection -> driver_dbc );

            if ( ret )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: IM005" );

                __post_internal_error( &connection -> error,
                        ERROR_IM005, NULL,
                        connection -> environment -> requested_version );
                return 0;
            }
        }
        else if ( CHECK_SQLALLOCHANDLE( connection ))
        {
            ret = SQLALLOCHANDLE( connection,
                    SQL_HANDLE_DBC,
                    connection -> driver_env,
                    &connection -> driver_dbc,
                    connection );

            if ( ret )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: IM005" );

                __post_internal_error( &connection -> error,
                        ERROR_IM005, NULL,
                        connection -> environment -> requested_version );
                return 0;
            }
        }
        else
        {
            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    "Error: IM005" );

            __post_internal_error( &connection -> error,
                    ERROR_IM005, NULL,
                    connection -> environment -> requested_version );
            return 0;
        }
    }

    /*
     * set any connection atributes
     */

    if ( connection -> login_timeout_set )
    {
        if (CHECK_SQLSETCONNECTATTR( connection ))
        {
            ret = SQLSETCONNECTATTR(connection,
                        connection -> driver_dbc,
                        SQL_ATTR_LOGIN_TIMEOUT,
                        connection -> login_timeout,
                        sizeof(connection -> login_timeout));
        }
        else if (CHECK_SQLSETCONNECTOPTION(connection))
        {
            ret = SQLSETCONNECTOPTION(connection,
                        connection -> driver_dbc,
                        SQL_LOGIN_TIMEOUT,
                        connection -> login_timeout );
        }
    }
    if ( connection -> auto_commit_set )
    {
        if (CHECK_SQLSETCONNECTATTR( connection ))
        {
            ret = SQLSETCONNECTATTR(connection,
                        connection -> driver_dbc,
                        SQL_ATTR_AUTOCOMMIT,
                        connection -> auto_commit,
                        sizeof(connection -> auto_commit));
        }
        else if (CHECK_SQLSETCONNECTOPTION(connection))
        {
            ret = SQLSETCONNECTOPTION(connection,
                        connection -> driver_dbc,
                        SQL_AUTOCOMMIT,
                        connection -> auto_commit );
        }
    }

    return 1;
}

/*
 * extract the available functions and call SQLSetConnectAttr
 */

int __connect_part_two( DMHDBC connection )
{
    int i, use_cursor;
    SQLCHAR s0[ 20 ];

    /*
     * Call SQLFunctions to get the supported list and
     * mask out those that are exported but not supported
     */

    if ( CHECK_SQLGETFUNCTIONS( connection ))
    {
        for ( i = 0;
            i < sizeof( template_func ) / sizeof( template_func[ 0 ] );
            i ++ )
        {
            if ( connection -> functions[ i ].func )
            {
                SQLRETURN ret;
                SQLUSMALLINT supported;

                if ( i > 100 )
                {
                    supported = FALSE;
                }
                else
                {
                    ret = SQLGETFUNCTIONS( connection,
                            connection -> driver_dbc,
                            connection -> functions[ i ].ordinal,
                            &supported );
                }

                if ( supported == SQL_FALSE )
                {
                    connection -> functions[ i ].func = NULL;
                    connection -> functions[ i ].can_supply = 0;
                }
            }
        }
    }

    /*
     * CoLAttributes is the same as ColAttribute
     */

    if ( connection -> functions[ DM_SQLCOLATTRIBUTE ].func &&
        !connection -> functions[ DM_SQLCOLATTRIBUTES ].func )
    {
        connection -> functions[ DM_SQLCOLATTRIBUTES ].can_supply = 1;
    }
    if ( connection -> functions[ DM_SQLCOLATTRIBUTES ].func &&
        !connection -> functions[ DM_SQLCOLATTRIBUTE ].func )
    {
        connection -> functions[ DM_SQLCOLATTRIBUTE ].can_supply = 1;
    }

    /*
     * mark the functions that the driver manager does
     */

    /*
     * SQLDatasources
     */
    connection -> functions[ DM_SQLDATASOURCES ].can_supply = 1;

    /*
     * SQLDrivers
     */
    connection -> functions[ DM_SQLDRIVERS ].can_supply = 1;

    /*
     * SQLAllocHandleStd
     */
    connection -> functions[ DM_SQLALLOCHANDLESTD ].can_supply = 1;

    /*
     * add all the functions that are supported via ODBC 2<->3
     * issues
     */
    if ( !connection -> functions[ DM_SQLALLOCENV ].func &&
            connection -> functions[ DM_SQLALLOCHANDLE ].func )
    {
        connection -> functions[ DM_SQLALLOCENV ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLALLOCCONNECT ].func &&
            connection -> functions[ DM_SQLALLOCHANDLE ].func )
    {
        connection -> functions[ DM_SQLALLOCCONNECT ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLALLOCSTMT ].func &&
            connection -> functions[ DM_SQLALLOCHANDLE ].func )
    {
        connection -> functions[ DM_SQLALLOCSTMT ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLFREEENV ].func &&
            connection -> functions[ DM_SQLFREEHANDLE ].func )
    {
        connection -> functions[ DM_SQLFREEENV ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLFREECONNECT ].func &&
            connection -> functions[ DM_SQLFREEHANDLE ].func )
    {
        connection -> functions[ DM_SQLALLOCCONNECT ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLGETDIAGREC ].func &&
            connection -> functions[ DM_SQLERROR ].func )
    {
        connection -> functions[ DM_SQLGETDIAGREC ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLGETDIAGFIELD ].func &&
            connection -> functions[ DM_SQLERROR ].func )
    {
        connection -> functions[ DM_SQLGETDIAGFIELD ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLERROR ].func &&
            connection -> functions[ DM_SQLGETDIAGREC ].func )
    {
        connection -> functions[ DM_SQLERROR ].can_supply = 1;
    }

    /*
     * ODBC 3 still needs SQLFreeStmt
     */

    /*
     * this is only partial, as we can't support a descriptor alloc
     */
    if ( !connection -> functions[ DM_SQLALLOCHANDLE ].func &&
            connection -> functions[ DM_SQLALLOCENV ].func &&
            connection -> functions[ DM_SQLALLOCCONNECT ].func &&
            connection -> functions[ DM_SQLALLOCHANDLE ].func )
    {
        connection -> functions[ DM_SQLALLOCHANDLE ].can_supply = 1;
    }
    if ( !connection -> functions[ DM_SQLFREEHANDLE ].func &&
            connection -> functions[ DM_SQLFREEENV ].func &&
            connection -> functions[ DM_SQLFREECONNECT ].func &&
            connection -> functions[ DM_SQLFREEHANDLE ].func )
    {
        connection -> functions[ DM_SQLFREEHANDLE ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLBINDPARAM ].func &&
                        connection -> functions[ DM_SQLBINDPARAMETER ].func )
    {
        connection -> functions[ DM_SQLBINDPARAM ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLBINDPARAMETER ].func &&
                        connection -> functions[ DM_SQLBINDPARAM ].func )
    {
        connection -> functions[ DM_SQLBINDPARAMETER ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLGETCONNECTOPTION ].func &&
                        connection -> functions[ DM_SQLGETCONNECTATTR ].func )
    {
        connection -> functions[ DM_SQLGETCONNECTOPTION ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLGETCONNECTATTR ].func &&
                        connection -> functions[ DM_SQLGETCONNECTOPTION ].func )
    {
        connection -> functions[ DM_SQLGETCONNECTATTR ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLGETSTMTOPTION ].func &&
                        connection -> functions[ DM_SQLGETSTMTATTR ].func )
    {
        connection -> functions[ DM_SQLGETSTMTOPTION ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLGETSTMTATTR ].func &&
                        connection -> functions[ DM_SQLGETSTMTOPTION ].func )
    {
        connection -> functions[ DM_SQLGETSTMTATTR ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLPARAMOPTIONS ].func &&
                        connection -> functions[ DM_SQLSETSTMTATTR ].func )
    {
        connection -> functions[ DM_SQLPARAMOPTIONS ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLSETCONNECTOPTION ].func &&
                        connection -> functions[ DM_SQLSETCONNECTATTR ].func )
    {
        connection -> functions[ DM_SQLSETCONNECTOPTION ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLSETCONNECTATTR ].func &&
                        connection -> functions[ DM_SQLSETCONNECTOPTION ].func )
    {
        connection -> functions[ DM_SQLSETCONNECTATTR ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLSETPARAM ].func &&
                        connection -> functions[ DM_SQLBINDPARAMETER ].func )
    {
        connection -> functions[ DM_SQLSETPARAM ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLSETSCROLLOPTIONS ].func &&
                        connection -> functions[ DM_SQLSETSTMTATTR ].func )
    {
        connection -> functions[ DM_SQLSETSCROLLOPTIONS ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLSETSTMTOPTION ].func &&
                        connection -> functions[ DM_SQLSETSTMTATTR ].func )
    {
        connection -> functions[ DM_SQLSETSTMTOPTION ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLSETSTMTATTR ].func &&
                        connection -> functions[ DM_SQLSETSTMTOPTION ].func )
    {
        connection -> functions[ DM_SQLSETSTMTATTR ].can_supply = 1;
    }

    if ( !connection -> functions[ DM_SQLTRANSACT ].func &&
                        connection -> functions[ DM_SQLENDTRAN ].func )
    {
        connection -> functions[ DM_SQLTRANSACT ].can_supply = 1;
    }
    else if ( !connection -> functions[ DM_SQLENDTRAN ].func &&
                        connection -> functions[ DM_SQLTRANSACT ].func )
    {
        connection -> functions[ DM_SQLENDTRAN ].can_supply = 1;
    }

    /*
     * we can always do this
     */

    if ( !connection -> functions[ DM_SQLGETFUNCTIONS ].func )
    {
        connection -> functions[ DM_SQLGETFUNCTIONS ].can_supply = 1;
    }

    /*
     * TO_DO get some driver settings, such as the GETDATA_EXTENSTION
     * it supports
     */

    /*
     * see if it's a version 3 driver so we can do descriptor
     * stuff.
     */

    connection -> driver_act_ver = 0;
    if ( CHECK_SQLGETINFO( connection ))
    {
        char txt[ 20 ];
        SQLRETURN ret;

        ret = SQLGETINFO( connection,
                    connection -> driver_dbc,
                    SQL_DRIVER_ODBC_VER,
                    txt,
                    sizeof( txt ),
                    NULL );

        if ( SQL_SUCCEEDED( ret ))
        {
            connection -> driver_act_ver = atoi( txt );
        }
    }

    /*
     * TO_DO now we should pass any SQLSetEnvAttr settings
     */

    /*
     * now we have a connection handle, and we can check to see if
     * we need to use the cursor library
     */

    if ( connection -> cursors == SQL_CUR_USE_ODBC )
    {
        use_cursor = 1;
    }
    else if ( connection -> cursors == SQL_CUR_USE_IF_NEEDED )
    {
        /*
         * get scrollable info
         */

        if ( !CHECK_SQLGETINFO( connection ))
        {
            /*
             * bit of a retarded driver, better give up
             */
            use_cursor = 0;
        }
        else
        {
            SQLRETURN ret;
            SQLUINTEGER val;

            /*
             * check if static cursors support scrolling
             */

            if ( connection -> driver_act_ver ==
                    SQL_OV_ODBC3 )
            {
                ret = SQLGETINFO( connection,
                        connection -> driver_dbc,
                        SQL_STATIC_CURSOR_ATTRIBUTES1,
                        &val,
                        NULL,
                        NULL );

                if ( ret != SQL_SUCCESS )
                {
                    use_cursor = 1;
                }
                else
                {
                    /*
                     * do we need it ?
                     */
                    if ( !( val & SQL_CA1_ABSOLUTE )) 
                    {
                        use_cursor = 1;
                    }
                    else
                    {
                        use_cursor = 0;
                    }
                }
            }
            else
            {
                ret = SQLGETINFO( connection,
                        connection -> driver_dbc,
                        SQL_FETCH_DIRECTION,
                        &val,
                        NULL,
                        NULL );

                if ( ret != SQL_SUCCESS )
                {
                    use_cursor = 1;
                }
                else
                {
                    /*
                     * are we needed
                     */

                    if ( !( val & SQL_FD_FETCH_PRIOR )) 
                    {
                        use_cursor = 1;
                    }
                    else 
                    {
                        use_cursor = 0;
                    }
                }
            }
        }
    }
    else
    {
        use_cursor = 0;
    }

    /*
     * if required connect to the cursor lib
     */

    if ( use_cursor )
    {
        int (*cl_connect)(void*);

        if ( !(connection -> cl_handle = lt_dlopen( CURSOR_LIB )))
        {
            char txt[ 256 ];

            sprintf( txt, "Can't open cursor lib '%s' : %s", 
                CURSOR_LIB, lt_dlerror());

            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    txt  );

            __post_internal_error( &connection -> error,
                    ERROR_01000, txt,
                    connection -> environment -> requested_version );

            return 0;
        }

        if ( !( cl_connect = (int(*)(void*))lt_dlsym( connection -> cl_handle,
                        "CLConnect" )))
        {
            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    "Error: 01000 Unable to load Cursor Lib" );

            __post_internal_error( &connection -> error,
                    ERROR_01000, "Unable to load cursor library",
                    connection -> environment -> requested_version );

            lt_dlclose( connection -> cl_handle );
            connection -> cl_handle = NULL;

            return 0;
        }
        if ( cl_connect( connection ) != SQL_SUCCESS )
        {
            lt_dlclose( connection -> cl_handle );
            connection -> cl_handle = NULL;
            return 0;
        }
    }
    else
    {
        connection -> cl_handle = NULL;
    }

    return 1;
}

/*
 * clean up after the first part of the connect
 */

void __disconnect_part_one( DMHDBC connection )
{
    int ret;

    /*
     * try a version 3 disconnect first on the connection
     */
    if ( connection -> driver_dbc )
    {
        if ( connection -> driver_version == SQL_OV_ODBC3 )
        {
            if ( CHECK_SQLFREEHANDLE( connection ))
            {
                ret = SQLFREEHANDLE( connection,
                        SQL_HANDLE_DBC,
                        connection -> driver_dbc );

                if ( !ret )
                    connection -> driver_dbc = (SQLHANDLE)NULL;
            }
        }

        /*
         * is it still open ?
         */

        if (  connection -> driver_dbc &&
                CHECK_SQLFREECONNECT( connection ))
        {
            SQLFREECONNECT( connection,
                    connection -> driver_dbc );
        }
        else
        {
            /*
             * not a lot we can do here
             */
        }
    }
    connection -> driver_dbc = (SQLHANDLE)NULL;

    /*
     * now disconnect the environment
     */

    if ( connection -> driver_env )
    {
        if ( connection -> driver_version == SQL_OV_ODBC3 )
        {
            if ( CHECK_SQLFREEHANDLE( connection ))
            {
                ret = SQLFREEHANDLE( connection,
                        SQL_HANDLE_ENV,
                        connection -> driver_env );

                if ( !ret )
                    connection -> driver_env = (SQLHANDLE)NULL;
            }
        }

        /*
         * is it still open ?
         */

        if ( CHECK_SQLFREEENV( connection ))
        {
            SQLFREEENV( connection,
                    connection -> driver_env );
        }
        else
        {
            /*
             * not a lot we can do here
             */
        }
    }
    connection -> driver_env = (SQLHANDLE)NULL;

    /*
     * unload the lib
     */

    if ( connection -> cl_handle )
    {
        lt_dlclose( connection -> cl_handle );
        connection -> cl_handle = NULL;
    }

    if ( connection -> dl_handle )
    {
        if ( !connection -> dont_dlclose )
        {
            /* 
             * call fini function if found
             */

            if ( connection -> fini_func.func )
            {
                connection -> fini_func.func();
            }

            lt_dlclose( connection -> dl_handle );
        }
        connection -> dl_handle = NULL;
    }

    /*
     * free some memory
     */

    if ( connection -> functions )
    {
        free( connection -> functions );
        connection -> functions = NULL;
    }
}

void __disconnect_part_two( DMHDBC connection )
{
    if ( CHECK_SQLDISCONNECT( connection ))
    {
        SQLDISCONNECT( connection,
                connection -> driver_dbc );
    }
}

/*
 * interface for SQLGetFunctions
 */

void  __check_for_function( DMHDBC connection,
        SQLUSMALLINT function_id,
        SQLUSMALLINT *supported )
{
    int i;

    if ( function_id == SQL_API_ODBC3_ALL_FUNCTIONS )
    {
        for ( i = 0; i < SQL_API_ODBC3_ALL_FUNCTIONS_SIZE; i ++ )
        {
            supported[ i ] = 0x0000;
        }
        for ( i = 0; i < sizeof( template_func ) / sizeof( template_func[ 0 ] ); i ++ )
        {
        int id = connection -> functions[ i ].ordinal;

            if ( connection -> functions[ i ].can_supply )
                supported[ id >> 4 ] |= ( 1 << ( id & 0x000F ));
        }
    }
    else if ( function_id == SQL_API_ALL_FUNCTIONS )
    {
        for ( i = 0; i < 100; i ++ )
        {
            supported[ i ] = SQL_FALSE;
        }
        for ( i = 0; i < sizeof( template_func ) / sizeof( template_func[ 0 ] ); i ++ )
        {
            if ( connection -> functions[ i ].ordinal < 100 )
            {
                if ( connection -> functions[ i ].can_supply )
                    supported[ connection -> functions[ i ].ordinal ] =
                        SQL_TRUE;
            }
        }
    }
    else
    {
        *supported = SQL_FALSE;
        for ( i = 0; i < sizeof( template_func ) / sizeof( template_func[ 0 ] ); i ++ )
        {
            if ( connection->functions[ i ].ordinal == function_id )
            {
                if ( connection -> functions[ i ].can_supply )
                    *supported = SQL_TRUE;
                break;
            }
        }
    }
}

static int sql_strcmp( SQLCHAR *s1, SQLCHAR *s2, SQLSMALLINT l1, SQLSMALLINT l2 )
{
    if ( l1 != l2 )
    {
        return 1;
    }

    if ( l1 == SQL_NTS )
    {
        return strcmp( s1, s2 );
    }
    else
    {
        return memcmp( s1, s1, l1 );
    }
}

static void close_pooled_connection( CPOOL *ptr )
{
    SQLRETURN ret;

    /*
     * disconnect from the driver
     */

    if ( !CHECK_SQLDISCONNECT(( &ptr->connection )))
    {
        return;
    }

    ret = SQLDISCONNECT(( &ptr -> connection ),
            &ptr -> connection.driver_dbc );

    if ( SQL_SUCCEEDED( ret ))
    {
        /*
         * complete disconnection from driver
         */

        if ( ptr -> connection.driver_version == SQL_OV_ODBC3 )
        {
            if ( CHECK_SQLFREEHANDLE(( &ptr -> connection )))
            {
                SQLFREEHANDLE(( &ptr -> connection ),
                        SQL_HANDLE_DBC,
                        ptr -> connection.driver_dbc );
            }
            else if ( CHECK_SQLFREECONNECT(( &ptr -> connection )))
            {
                SQLFREECONNECT(( &ptr -> connection ),
                        ptr -> connection.driver_dbc );
            }
        }
        else
        {
            if ( CHECK_SQLFREECONNECT(( &ptr -> connection )))
            {
                SQLFREECONNECT(( &ptr -> connection ),
                        ptr -> connection.driver_dbc );
            }
            else if ( CHECK_SQLFREEHANDLE(( &ptr -> connection )))
            {
                SQLFREEHANDLE(( &ptr -> connection ),
                        SQL_HANDLE_DBC,
                        ptr -> connection.driver_dbc );
            }
        }

        ptr -> connection.driver_dbc = (SQLHANDLE)NULL;

        if ( ptr -> connection.driver_version == SQL_OV_ODBC3 )
        {
            if ( CHECK_SQLFREEHANDLE(( &ptr -> connection )))
            {
                SQLFREEHANDLE(( &ptr -> connection ),
                        SQL_HANDLE_ENV,
                        ptr -> connection.driver_env );
            }
            else if ( CHECK_SQLFREEENV(( &ptr -> connection )))
            {
                SQLFREEENV(( &ptr -> connection ),
                        ptr -> connection.driver_env );
            }
        }
        else
        {
            if ( CHECK_SQLFREEENV(( &ptr -> connection )))
            {
                SQLFREEENV(( &ptr -> connection ),
                        ptr -> connection.driver_env );
            }
            else if ( CHECK_SQLFREEHANDLE(( &ptr -> connection )))
            {
                SQLFREEHANDLE(( &ptr -> connection ),
                        SQL_HANDLE_ENV,
                        ptr -> connection.driver_env );
            }
        }

        ptr -> connection.driver_env = (SQLHANDLE)NULL;

        /*
         * unload the lib
         */

        if ( ptr -> connection.cl_handle )
        {
            lt_dlclose( ptr -> connection.cl_handle );
            ptr -> connection.cl_handle = NULL;
        }

        if ( ptr -> connection.dl_handle )
        {
            /*
             * this is safe, because the dlopen function will reuse the handle if we 
             * open the same lib again
             */
            if ( !ptr -> connection.dont_dlclose )
            {
                /* 
                 * call fini function if found
                 */

                if ( ptr -> connection.fini_func.func )
                {
                    ptr -> connection.fini_func.func();
                }

                lt_dlclose( ptr -> connection.dl_handle );
            }
            ptr -> connection.dl_handle = NULL;
        }

        /*
         * free some memory
         */

        if ( ptr -> connection.functions )
        {
            free( ptr -> connection.functions );
            ptr -> connection.functions = NULL;
        }
    }

    /*
     * now clean up any statements that are left about
     */

    __clean_stmt_from_dbc( &ptr -> connection );
    __clean_desc_from_dbc( &ptr -> connection );
}

int search_for_pool( DMHDBC connection,
           SQLCHAR *server_name,
           SQLSMALLINT name_length1,
           SQLCHAR *user_name,
           SQLSMALLINT name_length2,
           SQLCHAR *authentication,
           SQLSMALLINT name_length3,
           SQLCHAR *connect_string,
           SQLSMALLINT connect_string_length )
{
    time_t current_time;
    SQLINTEGER dead;
    CPOOL *ptr, *prev;
    static int ij = 0;

    mutex_pool_entry();

    current_time = time( NULL );

    /*
     * look in the list of connections for one that matches
     */

restart:;

    for( ptr = pool_head, prev = NULL; ptr; prev = ptr, ptr = ptr -> next )
    {
        if ( ptr -> in_use )
        {
            continue;
        }

        /*
         * has it expired ? Do some cleaning up first
         */

        if ( ptr -> expiry_time < current_time )
        {
            /*
             * disconnect and remove
             */

            close_pooled_connection( ptr );

            if ( prev )
            {
                pool_head = ptr -> next;
                free( ptr );
                goto restart;
            }
            else
            {
                pool_head = NULL;
                free( ptr );
                goto restart;
            }
        }

        /*
         * has the time-to-live got to one ?
         */

        if ( ptr -> ttl == 1 )
        {
            /*
             * disconnect and remove
             */

            close_pooled_connection( ptr );

            if ( prev )
            {
                pool_head = ptr -> next;
                free( ptr );
                goto restart;
            }
            else
            {
                pool_head = NULL;
                free( ptr );
                goto restart;
            }
        }
        else if (  ptr -> ttl > 1 )
        {
            ptr -> ttl --;
        }

        /*
         * ok so far, is it still alive ?
         */

        if ( CHECK_SQLGETCONNECTATTR(( &ptr -> connection )))
        {
            SQLRETURN ret;

            ret = SQLGETCONNECTATTR(( &ptr -> connection ),
                ptr -> connection.driver_dbc,
                SQL_ATTR_CONNECTION_DEAD,
                &dead,
                0,
                0 );

            /*
             * if it failed assume that its because it doesn't support
             * it, but its ok
             */

            if ( SQL_SUCCEEDED( ret ))
            {
                if ( dead == SQL_CD_TRUE )
                {
                    /*
                     * disconnect and remove
                     */

                    close_pooled_connection( ptr );

                    if ( prev )
                    {
                        pool_head = ptr -> next;
                        free( ptr );
                        goto restart;
                    }
                    else
                    {
                        pool_head = NULL;
                        free( ptr );
                        goto restart;
                    }
                }
            }
        }

        if ( server_name )
        {
            if ( ptr -> server_length == 0 )
            {
                continue;
            }
            if ( ptr -> server_length != name_length1 ||
                    sql_strcmp( server_name, ptr -> server, 
                        name_length1, ptr -> server_length ))
            {
                continue;
            }
            if ( ptr -> user_length != name_length2 ||
                    sql_strcmp( user_name, ptr -> user, 
                        name_length2, ptr -> user_length ))
            {
                continue;
            }
            if ( ptr -> password_length != name_length3 ||
                    sql_strcmp( authentication, ptr -> password,
                        name_length3, ptr -> password_length ))
            {
                continue;
            }
        }
        else 
        {
            if ( ptr -> dsn_length == 0 )
            {
                continue;
            }
            if ( ptr -> dsn_length != connect_string_length ||
                    sql_strcmp( connect_string, ptr -> driver_connect_string,
                        connect_string_length, ptr -> dsn_length ))
            {
                continue;
            }
        }

        /*
         * is it the same cursor usage ?
         */

        if ( ptr -> cursors != connection -> cursors )
        {
            continue;
        }

        /*
         * at this point we have something that should work, lets use it
         */

        ptr -> in_use = 1;
        ptr -> expiry_time = current_time + ptr -> timeout;
        connection -> pooling_timeout = ptr -> timeout;

        /*
         * copy all the info over
         */

        connection -> pooled_connection = ptr;

        connection -> state = ptr -> connection.state;
        connection -> dl_handle = ptr -> connection.dl_handle;
        connection -> functions = ptr -> connection.functions;
        connection -> driver_env = ptr -> connection.driver_env;
        connection -> driver_dbc = ptr -> connection.driver_dbc;
        connection -> driver_version = ptr -> connection.driver_version;
        connection -> driver_act_ver = ptr -> connection.driver_act_ver;
        connection -> statement_count = 0;
        connection -> access_mode = ptr -> connection.access_mode;
        connection -> autocommit_mode = ptr -> connection.autocommit_mode;
        connection -> login_timeout = ptr -> connection.login_timeout;
        connection -> login_timeout_set = ptr -> connection.login_timeout_set;
        connection -> cursors = ptr -> connection.cursors;
        connection -> cl_handle = ptr -> connection.cl_handle;

#ifdef HAVE_LIBPTHREAD
        connection -> mutex = ptr -> connection.mutex;
        connection -> protection_level = ptr -> connection.protection_level;
#elif HAVE_LIBTHREAD
        connection -> mutex = ptr -> connection.mutex;
        connection -> protection_level = ptr -> connection.protection_level;
#endif

        /*
         * TODO
         * alter the mutex level here
         */

        connection -> ex_fetch_mapping = ptr -> connection.ex_fetch_mapping;
        connection -> dont_dlclose = ptr -> connection.dont_dlclose;
        connection -> bookmarks_on = ptr -> connection.bookmarks_on;

        strcpy( connection -> dsn, ptr -> connection.dsn );

        mutex_pool_exit();

        return TRUE;
    }

    mutex_pool_exit();
    return FALSE;
}

void return_to_pool( DMHDBC connection )
{
    CPOOL *ptr;
    time_t current_time;

    mutex_pool_entry();

    ptr = connection -> pooled_connection;
    current_time = time( NULL );

    /*
     * is it a old entry ?
     */

    if ( connection -> pooled_connection )
    {
        ptr -> in_use = 0;
        ptr -> expiry_time = current_time + ptr -> timeout;
    }
    else
    {
        ptr = calloc( sizeof( CPOOL ), 1 );
        if ( !ptr )
        {
            mutex_pool_exit();
            return;
        }

        /*
         * copy everything over
         */

        ptr -> in_use = 0;
        ptr -> expiry_time = current_time + connection -> pooling_timeout;
        ptr -> timeout = connection -> pooling_timeout;
        ptr -> ttl = connection -> ttl;
        ptr -> cursors = connection -> cursors;

        /*
         * copy all the info over
         */

        ptr -> connection.state = connection -> state;
        ptr -> connection.dl_handle = connection -> dl_handle;
        ptr -> connection.functions = connection -> functions;
        ptr -> connection.driver_env = connection -> driver_env;
        ptr -> connection.driver_dbc = connection -> driver_dbc;
        ptr -> connection.driver_version = connection -> driver_version;
        ptr -> connection.driver_act_ver = connection -> driver_act_ver;
        ptr -> connection.access_mode = connection -> access_mode;
        ptr -> connection.autocommit_mode = connection -> autocommit_mode;
        ptr -> connection.login_timeout = connection -> login_timeout;
        ptr -> connection.login_timeout_set = connection -> login_timeout_set;
        ptr -> connection.cursors = connection -> cursors;
        ptr -> connection.cl_handle = connection -> cl_handle;

#ifdef HAVE_LIBPTHREAD
        ptr -> connection.mutex = connection -> mutex;
        ptr -> connection.protection_level = connection -> protection_level;
#elif HAVE_LIBTHREAD
        ptr -> connection.mutex = connection -> mutex;
        ptr -> connection.protection_level = connection -> protection_level;
#endif

        ptr -> connection.pooling_timeout = ptr -> timeout;

        ptr -> connection.ex_fetch_mapping = connection -> ex_fetch_mapping;
        ptr -> connection.dont_dlclose = connection -> dont_dlclose;
        ptr -> connection.bookmarks_on = connection -> bookmarks_on;

        if ( connection -> server_length < 0 )
        {
            strcpy( ptr -> server, connection -> server );
        }
        else
        {
            memcpy( ptr -> server, connection -> server, connection -> server_length );
        }
        ptr -> server_length = connection -> server_length;

        if ( connection -> user_length < 0 )
        {
            strcpy( ptr -> user, connection -> user );
        }
        else
        {
            memcpy( ptr -> user, connection -> user, connection -> user_length );
        }
        ptr -> user_length = connection -> user_length;

        if ( connection -> password_length < 0 )
        {
            strcpy( ptr -> password, connection -> password );
        }
        else
        {
            memcpy( ptr -> password, connection -> password, connection -> password_length );
        }
        ptr -> password_length = connection -> password_length;

        if ( connection -> dsn_length < 0 )
        {
            strcpy( ptr -> driver_connect_string, connection -> driver_connect_string );
        }
        else
        {
            memcpy( ptr -> driver_connect_string, connection -> driver_connect_string,
                    connection -> dsn_length );
        }
        ptr -> dsn_length = connection -> dsn_length;

        strcpy( ptr -> connection.dsn, connection -> dsn );

        /*
         * add to the list
         */

        ptr -> next = pool_head;
        pool_head = ptr;
    }

    /*
     * remove all information from the connection
     */

    connection -> state = STATE_C2;
    connection -> driver_env = 0;
    connection -> driver_dbc = 0;
    connection -> dl_handle = 0;
    connection -> cl_handle = 0;
    connection -> functions = 0;
    connection -> pooled_connection = 0;

    mutex_pool_exit();
}

SQLRETURN SQLConnect( SQLHDBC connection_handle,
           SQLCHAR *server_name,
           SQLSMALLINT name_length1,
           SQLCHAR *user_name,
           SQLSMALLINT name_length2,
           SQLCHAR *authentication,
           SQLSMALLINT name_length3 )
{
    DMHDBC connection = (DMHDBC)connection_handle;
    int len, ret_from_connect;
    char dsn[ SQL_MAX_DSN_LENGTH + 1 ];
    char lib_name[ INI_MAX_PROPERTY_VALUE + 1 ];
    char driver_name[ INI_MAX_PROPERTY_VALUE + 1 ];
    SQLCHAR s1[ 100 ], s2[ 100 ], s3[ 100 ];

    /*
     * check connection
     */
    if ( !__validate_dbc( connection ))
    {
        dm_log_write( __FILE__, 
                __LINE__, 
                    LOG_INFO, 
                    LOG_INFO, 
                    "Error: SQL_INVALID_HANDLE" );

        return SQL_INVALID_HANDLE;
    }

    function_entry( connection );

    if ( log_info.log_flag )
    {
        sprintf( connection -> msg, "\n\t\tEntry:\
            \n\t\t\tConnection = %p\
            \n\t\t\tServer Name = %s\
            \n\t\t\tUser Name = %s\
            \n\t\t\tAuthentication = %s",
                connection,
                __string_with_length( s1, server_name, name_length1 ),
                __string_with_length( s2, user_name, name_length2 ),
                __string_with_length_pass( s3, authentication, name_length3 ));

        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                connection -> msg );
    }

    thread_protect( SQL_HANDLE_DBC, connection );

    if (( name_length1 < 0 && name_length1 != SQL_NTS ) ||
        ( name_length2 < 0 && name_length2 != SQL_NTS ) ||
        ( name_length3 < 0 && name_length3 != SQL_NTS ))

    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: HY090" );

        __post_internal_error( &connection -> error,
                ERROR_HY090, NULL,
                connection -> environment -> requested_version );

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, SQL_ERROR );
    }

    /*
     * check the state of the connection
     */
    if ( connection -> state != STATE_C2 )
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: 08002" );

        __post_internal_error( &connection -> error,
                ERROR_08002, NULL,
                connection -> environment -> requested_version );

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, SQL_ERROR );
    }

    if ( name_length1 && server_name )
    {
        if ( name_length1 == SQL_NTS )
        {
            len = strlen( server_name );

            if ( len > SQL_MAX_DSN_LENGTH )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: HY090" );

                __post_internal_error( &connection -> error,
                        ERROR_HY090, NULL,
                        connection -> environment -> requested_version );

                thread_release( SQL_HANDLE_DBC, connection );

                return function_return( connection, SQL_ERROR );
            }
        }
        else
        {
            len = name_length1;

            if ( len > SQL_MAX_DSN_LENGTH )
            {
                dm_log_write( __FILE__,
                        __LINE__,
                        LOG_INFO,
                        LOG_INFO,
                        "Error: HY090" );

                __post_internal_error( &connection -> error,
                        ERROR_HY090, NULL,
                        connection -> environment -> requested_version );

                thread_release( SQL_HANDLE_DBC, connection );

                return function_return( connection, SQL_ERROR );
            }
        }

        memcpy( dsn, server_name, len );
        dsn[ len ] ='\0';
    }
    else if ( name_length1 && !server_name )
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: IM010" );

        __post_internal_error( &connection -> error,
                ERROR_IM010, NULL,
                connection -> environment -> requested_version );

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, SQL_ERROR );
    }
    else
    {
        strcpy( dsn, "DEFAULT" );
    }

    /*
     * can we find a pooled connection to use here ?
     */

    connection -> pooled_connection = NULL;

    if ( pooling_enabled && search_for_pool( connection, 
                                                server_name, name_length1,
                                                user_name, name_length2,
                                                authentication, name_length3,
                                                NULL, 0 ))
    {
        ret_from_connect = SQL_SUCCESS;

        if ( log_info.log_flag )
        {
            sprintf( connection -> msg,
                    "\n\t\tExit:[%s]",
                        __get_return_status( ret_from_connect ));

            dm_log_write( __FILE__,
                        __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    connection -> msg );
        }

        connection -> state = STATE_C4;

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, ret_from_connect );
    }

    /*
     * else safe the info for later
     */

    if ( pooling_enabled )
    {
        connection -> dsn_length = 0;

        if ( server_name )
        {
            if ( name_length1 < 0 )
            {
                strcpy( connection -> server, server_name );
            }
            else
            {
                memcpy( connection -> server, server_name, name_length1 );
            }
        }
        else
        {
            strcpy( connection -> server, "" );
        }
        connection -> server_length = name_length1;

        if ( user_name )
        {
            if ( name_length2 < 0 )
            {
                strcpy( connection -> user, user_name );
            }
            else
            {
                memcpy( connection -> user, user_name, name_length2 );
            }
        }
        else
        {
            strcpy( connection -> user, "" );
        }
        connection -> user_length = name_length2;

        if ( authentication )
        {
            if ( name_length3 )
            {
                strcpy( connection -> password, authentication );
            }
            else
            {
                memcpy( connection -> password, authentication, name_length3 );
            }
        }
        else
        {
            strcpy( connection -> password, "" );
        }
        connection -> password_length = name_length3;
    }

    if ( !__find_lib_name( dsn, lib_name, driver_name ))
    {
        /*
         * if not found look for a default
         */

        if ( !__find_lib_name( "DEFAULT", lib_name, driver_name ))
        {
            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    "Error: IM002" );

            __post_internal_error( &connection -> error,
                    ERROR_IM002, NULL,
                    connection -> environment -> requested_version );

            thread_release( SQL_HANDLE_DBC, connection );

            return function_return( connection, SQL_ERROR );
        }
    }

    /*
     * if necessary change the threading level
     */

    if ( !__connect_part_one( connection, lib_name, driver_name ))
    {
        thread_release( SQL_HANDLE_DBC, connection );
        return function_return( connection, SQL_ERROR );
    }

    if ( !CHECK_SQLCONNECT( connection ) &&
        !CHECK_SQLCONNECTW( connection ))
    {
        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                "Error: IM001" );

        __disconnect_part_one( connection );
        __post_internal_error( &connection -> error,
                ERROR_IM001, NULL,
                connection -> environment -> requested_version );

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, SQL_ERROR );
    }

    if ( CHECK_SQLCONNECT( connection ))
    {
        ret_from_connect = SQLCONNECT( connection,
                connection -> driver_dbc,
                dsn, SQL_NTS,
                user_name, name_length2,
                authentication, name_length3 );

        if ( ret_from_connect != SQL_SUCCESS )
        {
            SQLCHAR sqlstate[ 6 ];
            SQLINTEGER native_error;
            SQLSMALLINT ind;
            SQLCHAR message_text[ SQL_MAX_MESSAGE_LENGTH + 1 ];
            SQLRETURN ret;

            /*
             * get the errors from the driver before
             * loseing the connection 
             */

            if ( CHECK_SQLERROR( connection ))
            {
                do
                {
                    ret = SQLERROR( connection,
                            SQL_NULL_HENV,
                            connection -> driver_dbc,
                            SQL_NULL_HSTMT,
                            sqlstate,
                            &native_error,
                            message_text,
                            sizeof( message_text ),
                            &ind );


                    if ( SQL_SUCCEEDED( ret ))
                    {
                        __post_internal_error_ex( &connection -> error,
                                sqlstate,
                                native_error,
                                message_text );

                        sprintf( connection -> msg, "\t\tDIAG [%s] %s",
                                sqlstate, message_text );

                        dm_log_write_diag( connection -> msg );
                    }
                }
                while( SQL_SUCCEEDED( ret ));
            }
            else if ( CHECK_SQLGETDIAGREC( connection ))
            {
                int rec = 1;

                do
                {
                    ret = SQLGETDIAGREC( connection,
                            SQL_HANDLE_DBC,
                            connection -> driver_dbc,
                            rec ++,
                            sqlstate,
                            &native_error,
                            message_text,
                            sizeof( message_text ),
                            &ind );


                    if ( SQL_SUCCEEDED( ret ))
                    {
                        __post_internal_error_ex( &connection -> error,
                                sqlstate,
                                native_error,
                                message_text );

                        sprintf( connection -> msg, "\t\tDIAG [%s] %s",
                            sqlstate, message_text );

                        dm_log_write_diag( connection -> msg );
                    }
                }
                while( SQL_SUCCEEDED( ret ));
            }
        }

        /* 
         * if it was a error then return now
         */

        if ( !SQL_SUCCEEDED( ret_from_connect ))
        {
            __disconnect_part_one( connection );

            thread_release( SQL_HANDLE_DBC, connection );

            sprintf( connection -> msg,
                    "\n\t\tExit:[%s]",
                        __get_return_status( ret_from_connect ));

            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    connection -> msg );

            return function_return( connection, ret_from_connect );
        }

        connection -> unicode_driver = 0;
    }
    else
    {
        SQLWCHAR * uc_dsn, *uc_user, *uc_auth;

        uc_dsn = ansi_to_unicode( dsn, SQL_NTS );
        uc_user = ansi_to_unicode( user_name, name_length2 );
        uc_auth = ansi_to_unicode( authentication, name_length3 );

        ret_from_connect = SQLCONNECTW( connection,
                connection -> driver_dbc,
                uc_dsn, SQL_NTS,
                uc_user, name_length2,
                uc_auth, name_length3 );

        if ( uc_dsn )
            free( uc_dsn );
        if ( uc_user )
            free( uc_user );
        if ( uc_auth )
            free( uc_auth );

        if ( ret_from_connect != SQL_SUCCESS )
        {
            SQLWCHAR sqlstate[ 6 ];
            SQLINTEGER native_error;
            SQLSMALLINT ind;
            SQLWCHAR message_text[ SQL_MAX_MESSAGE_LENGTH + 1 ];
            SQLRETURN ret;

            /*
             * get the errors from the driver before
             * loseing the connection 
             */

            if ( CHECK_SQLERRORW( connection ))
            {
                do
                {
                    ret = SQLERRORW( connection,
                            SQL_NULL_HENV,
                            connection -> driver_dbc,
                            SQL_NULL_HSTMT,
                            sqlstate,
                            &native_error,
                            message_text,
                            sizeof( message_text ),
                            &ind );


                    if ( SQL_SUCCEEDED( ret ))
                    {
                        __post_internal_error_ex_w( &connection -> error,
                                sqlstate,
                                native_error,
                                message_text );

                        unicode_to_ansi( sqlstate, SQL_NTS );
                        unicode_to_ansi( message_text, SQL_NTS );

                        sprintf( connection -> msg, "\t\tDIAG [%s] %s",
                                sqlstate, message_text );

                        dm_log_write_diag( connection -> msg );
                    }
                }
                while( SQL_SUCCEEDED( ret ));
            }
            else if ( CHECK_SQLGETDIAGRECW( connection ))
            {
                int rec = 1;

                do
                {
                    ret = SQLGETDIAGRECW( connection,
                            SQL_HANDLE_DBC,
                            connection -> driver_dbc,
                            rec ++,
                            sqlstate,
                            &native_error,
                            message_text,
                            sizeof( message_text ),
                            &ind );

                    if ( SQL_SUCCEEDED( ret ))
                    {
                        __post_internal_error_ex_w( &connection -> error,
                                sqlstate,
                                native_error,
                                message_text );

                        unicode_to_ansi( sqlstate, SQL_NTS );
                        unicode_to_ansi( message_text, SQL_NTS );

                        sprintf( connection -> msg, "\t\tDIAG [%s] %s",
                                sqlstate, message_text );

                        dm_log_write_diag( connection -> msg );
                    }
                }
                while( SQL_SUCCEEDED( ret ));
            }
        }

        /* 
         * if it was a error then return now
         */

        if ( !SQL_SUCCEEDED( ret_from_connect ))
        {
            __disconnect_part_one( connection );

            thread_release( SQL_HANDLE_DBC, connection );

            sprintf( connection -> msg,
                    "\n\t\tExit:[%s]",
                        __get_return_status( ret_from_connect ));

            dm_log_write( __FILE__,
                    __LINE__,
                    LOG_INFO,
                    LOG_INFO,
                    connection -> msg );

            return function_return( connection, ret_from_connect );
        }

        connection -> unicode_driver = 1;
    }

    /*
     * we should be connected now
     */
    connection -> state = STATE_C4;
    strcpy( connection -> dsn, dsn );

    /*
     * did we get the type we wanted
     */
    if ( connection -> driver_version !=
            connection -> environment -> requested_version )
    {
        connection -> driver_version =
            connection -> environment -> requested_version;

        __post_internal_error( &connection -> error,
                ERROR_01000, "Driver does not support the requested version",
                connection -> environment -> requested_version );
        ret_from_connect = SQL_SUCCESS_WITH_INFO;
    }

    if ( !__connect_part_two( connection ))
    {
        /*
         * the cursor lib can kill us here, so be careful
         */

        __disconnect_part_two( connection );
        __disconnect_part_one( connection );

        connection -> state = STATE_C3;

        thread_release( SQL_HANDLE_DBC, connection );

        return function_return( connection, SQL_ERROR );
    }

    if ( log_info.log_flag )
    {
        sprintf( connection -> msg,
                "\n\t\tExit:[%s]",
                    __get_return_status( ret_from_connect ));

        dm_log_write( __FILE__,
                __LINE__,
                LOG_INFO,
                LOG_INFO,
                connection -> msg );
    }

    thread_release( SQL_HANDLE_DBC, connection );

    return function_return( connection, ret_from_connect );
}
