/**************************************************
 *
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 31.JAN.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include "classODBCConfig.h"

#include "smallLinuxODBC.xpm"

classODBCConfig::classODBCConfig( QWidget* parent, const char* name )
	: QWidget( parent, name )
{
	tbTabBar = new QTabBar( this, "tbTabBar" );
	tbTabBar->setGeometry( 0, 0, 430, 30 );
	tbTabBar->setMinimumSize( 0, 0 );
	tbTabBar->setMaximumSize( 32767, 32767 );
	connect( tbTabBar, SIGNAL(selected(int)), SLOT(TabChanged(int)) );
	tbTabBar->setFocusPolicy( QWidget::TabFocus );
	tbTabBar->setBackgroundMode( QWidget::PaletteBackground );
	tbTabBar->setShape( QTabBar::RoundedAbove );
	{
		QTab* tab;
		tab = new QTab( "&User DSN" );
		tbTabBar->addTab( tab );

		tab = new QTab( "&System DSN" );
		tbTabBar->addTab( tab );

		tab = new QTab( "&Drivers" );
		tbTabBar->addTab( tab );

		tab = new QTab( "St&ats" );
		tbTabBar->addTab( tab );

		tab = new QTab( "Ad&vanced" );
		tbTabBar->addTab( tab );

		tab = new QTab( "&About" );
		tbTabBar->addTab( tab );

	}

    fraTabFrame = new QFrame( this, "fraTabFrame" );
    fraTabFrame->setGeometry( 0, 30, 390, 300 );
	fraTabFrame->setMinimumSize( 0, 0 );
	fraTabFrame->setMaximumSize( 32767, 32767 );
	fraTabFrame->setFocusPolicy( QWidget::NoFocus );
	fraTabFrame->setBackgroundMode( QWidget::PaletteBackground );
	fraTabFrame->setFrameStyle( 34 );

	resize( 500,330 );
	setMinimumSize( 0, 0 );
	setMaximumSize( 32767, 32767 );
   
    setIcon( QPixmap( smallLinuxODBC_xpm ) );
    setCaption( "ODBC Data Source Administrator " VERSION );
    pUserDSN       = new classUserDSN( fraTabFrame );
    pSystemDSN     = new classSystemDSN( fraTabFrame );
    pDrivers       = new classDrivers( fraTabFrame );
    pStats         = new classStats( fraTabFrame );
    pTracing       = new classTracing( fraTabFrame );
    pAbout         = new classAbout( fraTabFrame );
   
    LoadState();
}


classODBCConfig::~classODBCConfig()
{
    SaveState();
}

void classODBCConfig::resizeEvent( QResizeEvent *p )
{
    resize( p->size() );
    tbTabBar->resize( p->size().width (), tbTabBar->height() );
    fraTabFrame->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pUserDSN->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pSystemDSN->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pDrivers->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pStats->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pTracing->resize( p->size().width (), p->size().height() - tbTabBar->height() );
    pAbout->resize( p->size().width (), p->size().height() - tbTabBar->height() );
}

void classODBCConfig::TabChanged( int nTab )
{
    QString qsError;

    pUserDSN->hide();
    pSystemDSN->hide();
    pDrivers->hide();
    pStats->hide();
	pTracing->hide();
    pAbout->hide();

    qsError.sprintf( "nTab=%d", nTab );

    switch ( nTab )
    {
    case TAB_USERDSN:
        pUserDSN->show();
        break;
    case TAB_SYSTEMDSN:
        pSystemDSN->show();
        break;
    case TAB_DRIVERS:
        pDrivers->show();
        break;
    case TAB_STATS:
        pStats->show();
        break;
    case TAB_TRACING:
        pTracing->show();
        break;
    case TAB_ABOUT:
        pAbout->show();
        break;
    default:
        QMessageBox::information( this, "ODBC Config", "Unknown tab " + qsError );
    }
}

void classODBCConfig::LoadState()
{
	HINI hIni;
	char szObject[INI_MAX_OBJECT_NAME+1];
	char szPropertyName[INI_MAX_PROPERTY_NAME+1];
	char szPropertyValue[INI_MAX_PROPERTY_VALUE+1];
    char szINI[ODBC_FILENAME_MAX+1];

    sprintf( szINI, "%s/%s", QDir::homeDirPath().ascii(), ".ODBCConfig/ODBCConfig.ini" ); 
	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
	{
		iniObjectFirst( hIni );
		while ( iniObjectEOL( hIni ) == FALSE )
		{
			szObject[0]         = '\0';
			szPropertyName[0] 	= '\0';
			szPropertyValue[0] 	= '\0';
			iniObject( hIni, szObject );
			iniPropertyFirst( hIni );
            if ( strcmp( szObject, "GEOMETRY" ) == 0 )
            {
                int nX = x();
                int nY = y();
                int nW = width();
                int nH = height();
                int nTab = 0;

    			while ( iniPropertyEOL( hIni ) == FALSE )
    			{
    				iniProperty( hIni, szPropertyName );
    				if ( strncmp( szPropertyName, "X", INI_MAX_PROPERTY_NAME ) == 0 )
                    {
    					iniValue( hIni, szPropertyValue );
                        nX = atoi( szPropertyValue );
                    }
    				if ( strncmp( szPropertyName, "Y", INI_MAX_PROPERTY_NAME ) == 0 )
                    {
    					iniValue( hIni, szPropertyValue );
                        nY = atoi( szPropertyValue );
                    }
    				if ( strncmp( szPropertyName, "W", INI_MAX_PROPERTY_NAME ) == 0 )
                    {
    					iniValue( hIni, szPropertyValue );
                        nW = atoi( szPropertyValue );
                    }
    				if ( strncmp( szPropertyName, "H", INI_MAX_PROPERTY_NAME ) == 0 )
                    {
    					iniValue( hIni, szPropertyValue );
                        nH = atoi( szPropertyValue );
                    }
    				if ( strncmp( szPropertyName, "Tab", INI_MAX_PROPERTY_NAME ) == 0 )
                    {
    					iniValue( hIni, szPropertyValue );
                        nTab = atoi( szPropertyValue );
                    }

    				iniPropertyNext( hIni );
    			}
                resize( nW, nH );
                move( nX, nY );
                tbTabBar->setCurrentTab( nTab );
            }
			iniObjectNext( hIni );
		}
	}
	else
	{
		QMessageBox::warning(	this, "ODBCConfig",  "Could not open ~/.ODBCConfig/ODBCConfig.ini.\nYou may not have a home dir." );
	}
}

void classODBCConfig::SaveState()
{
	HINI hIni;
	char szPropertyValue[INI_MAX_PROPERTY_VALUE+1];
    char szINI[ODBC_FILENAME_MAX+1];

    sprintf( szINI, "%s/%s", QDir::homeDirPath().ascii(), ".ODBCConfig/ODBCConfig.ini" ); 
	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
	{
        // REMOVE OBJECT
		iniObjectSeekSure( hIni, "GEOMETRY" );
        iniObjectDelete( hIni );

        // ADD OBJECT
		iniObjectSeekSure( hIni, "GEOMETRY" );
        sprintf( szPropertyValue, "%d", pos().x() );
        iniPropertyInsert( hIni, "X", szPropertyValue );

        sprintf( szPropertyValue, "%d", pos().y() );
        iniPropertyInsert( hIni, "Y", szPropertyValue );

        sprintf( szPropertyValue, "%d", size().width() );
        iniPropertyInsert( hIni, "W", szPropertyValue );

        sprintf( szPropertyValue, "%d", size().height() );
        iniPropertyInsert( hIni, "H", szPropertyValue );

        sprintf( szPropertyValue, "%d", tbTabBar->currentTab() );
        iniPropertyInsert( hIni, "Tab", szPropertyValue );

        iniCommit( hIni );
	}
}


